@@ -22,113 +22,8 @@
 
 #include <libgnomeui/libgnomeui.h>
 
-static gboolean PyGtk_FatalExceptions = FALSE;
-
-/* -- from other modules -- */
-typedef struct {
-  PyObject_HEAD
-  GtkObject *obj;
-} PyGtk_Object;
-static PyTypeObject *PyGtk_Type;
-
-typedef struct {
-  PyObject_HEAD
-  GtkAccelGroup *obj;
-} PyGtkAccelGroup_Object;
-static PyTypeObject *PyGtkAccelGroup_Type;
-
-typedef struct {
-  PyObject_HEAD
-  GdkFont *obj;
-} PyGdkFont_Object;
-static PyTypeObject *PyGdkFont_Type;
-
-typedef struct {
-  PyObject_HEAD
-  GdkColor obj;
-} PyGdkColor_Object;
-static PyTypeObject *PyGdkColor_Type;
-
-typedef struct {
-  PyObject_HEAD
-  GdkEvent *obj;
-        PyObject *attrs;
-} PyGdkEvent_Object;
-static PyTypeObject *PyGdkEvent_Type;
-
-typedef struct {
-  PyObject_HEAD
-  GdkWindow *obj;
-} PyGdkWindow_Object;
-static PyTypeObject *PyGdkWindow_Type;
-
-typedef struct {
-  PyObject_HEAD
-  GdkGC *obj;
-} PyGdkGC_Object;
-static PyTypeObject *PyGdkGC_Type;
-
-typedef struct {
-  PyObject_HEAD
-  GdkColormap *obj;
-} PyGdkColormap_Object;
-static PyTypeObject *PyGdkColormap_Type;
-
-typedef struct {
-    PyObject_HEAD
-    GdkCursor *obj;
-} PyGdkCursor_Object;
-static PyTypeObject *PyGdkCursor_Type;
-
-typedef struct {
-    PyObject_HEAD
-    GdkImlibImage *obj;
-} PyGdkImlibImage_Object;
-static PyTypeObject *PyGdkImlibImage_Type;
-
-#define PyGtk_Check(v) ((v)->ob_type == PyGtk_Type)
-#define PyGtk_Get(v) (((PyGtk_Object *)(v))->obj)
-#define PyGtkAccelGroup_Check(v) ((v)->ob_type == PyGtkAccelGroup_Type)
-#define PyGtkAccelGroup_Get(v) (((PyGtkAccelGroup_Object *)(v))->obj)
-#define PyGdkFont_Check(v) ((v)->ob_type == PyGdkFont_Type)
-#define PyGdkFont_Get(v) (((PyGdkFont_Object *)(v))->obj)
-#define PyGdkColor_Check(v) ((v)->ob_type == PyGdkColor_Type)
-#define PyGdkColor_Get(v) (&((PyGdkColor_Object *)(v))->obj)
-#define PyGdkEvent_Check(v) ((v)->ob_type == PyGdkEvent_Type)
-#define PyGdkEvent_Get(v) (((PyGdkEvent_Object *)(v))->obj)
-#define PyGdkWindow_Check(v) ((v)->ob_type == PyGdkWindow_Type)
-#define PyGdkWindow_Get(v) (((PyGdkWindow_Object *)(v))->obj)
-#define PyGdkGC_Check(v) ((v)->ob_type == PyGdkGC_Type)
-#define PyGdkGC_Get(v) (((PyGdkGC_Object *)(v))->obj)
-#define PyGdkColormap_Check(v) ((v)->ob_type == PyGdkColormap_Type)
-#define PyGdkColormap_Get(v) (((PyGdkColormap_Object *)(v))->obj)
-#define PyGdkCursor_Check(v) ((v)->ob_type == PyGdkCursor_Type)
-#define PyGdkCursor_Get(v) (((PyGdkCursor_Object *)(v))->obj)
-#define PyGdkImlibImage_Check(v) ((v)->ob_type == PyGdkImlibImage_Type)
-#define PyGdkImlibImage_Get(v) (((PyGdkImlibImage_Object *)(v))->obj)
-
-static PyObject *(*PyGtk_New)(GtkObject *)=0;
-static PyObject *(*PyGtkAccelGroup_New)(GtkAccelGroup *)=0;
-static PyObject *(*PyGdkFont_New)(GdkFont *)=0;
-static PyObject *(*PyGdkColor_New)(GdkColor *)=0;
-static PyObject *(*PyGdkEvent_New)(GdkEvent *)=0;
-static PyObject *(*PyGdkWindow_New)(GdkWindow *)=0;
-static PyObject *(*PyGdkGC_New)(GdkGC *)=0;
-static PyObject *(*PyGdkColormap_New)(GdkColormap *)=0;
-static PyObject *(*PyGdkCursor_New)(GdkCursor *)=0;
-static PyObject *(*PyGdkImlibImage_New)(GdkImlibImage *)=0;
-
-static GtkCallbackMarshal PyGtk_CallbackMarshal = 0;
-static GtkDestroyNotify PyGtk_DestroyNotify = 0;
-
-static int (*PyGtkEnum_get_value)(GtkType type, PyObject *obj, gint *val)=0;
-static int (*PyGtkFlag_get_value)(GtkType type, PyObject *obj, gint *val)=0;
-
-static GtkArg *(*PyDict_AsGtkArgs)(PyObject *dict, GtkType type, gint*nargs)=0;
-static void (*PyGtk_BlockThreads)(void);
-static void (*PyGtk_UnblockThreads)(void);
-
-/* --- gnomeui stuff --- */
+#include "../pygtk/pygtk.h"
+#include "../pygtk/pygdkimlib.h"
 
 /* This is needed because a NULL signal handler is taken to mean that the
  * python signal handler should be called using the user data as a function.
@@ -354,11 +249,6 @@ static int points_toarg(gpointer *boxed, PyObject *obj) {
   return 0;
 }
 
-typedef PyObject *(*fromargfunc)(gpointer boxed);
-typedef int (*toargfunc)(gpointer *boxed, PyObject *obj);
-static void (*PyGtk_RegisterBoxed)(GtkType boxed_type,
-				  fromargfunc fromarg, toargfunc toarg);
-
 static PyObject *_register_types() {
   static gboolean called = FALSE;
 
@@ -433,7 +323,7 @@ static PyObject *_wrap_gnome_app_create_menus(PyObject *self, PyObject *args) {
   PyObject *app, *list;
   GnomeUIInfo *uiinfo;
 
-  if (!PyArg_ParseTuple(args, "O!O:gnome_app_create_menus", PyGtk_Type, &app,
+  if (!PyArg_ParseTuple(args, "O!O:gnome_app_create_menus", &PyGtk_Type, &app,
 			&list))
     return NULL;
   if ((uiinfo = list_to_ui_info(list, FALSE)) == NULL)
@@ -449,7 +339,7 @@ static PyObject *_wrap_gnome_app_create_toolbar(PyObject *self, PyObject *args)
   PyObject *app, *list;
   GnomeUIInfo *uiinfo;
 
-  if (!PyArg_ParseTuple(args, "O!O:gnome_app_create_toolbar", PyGtk_Type, &app,
+  if (!PyArg_ParseTuple(args, "O!O:gnome_app_create_toolbar", &PyGtk_Type, &app,
 			&list))
     return NULL;
   if ((uiinfo = list_to_ui_info(list, FALSE)) == NULL)
@@ -466,7 +356,7 @@ static PyObject *_wrap_gnome_app_insert_menus(PyObject *self, PyObject *args) {
   char *path;
   GnomeUIInfo *uiinfo;
 
-  if (!PyArg_ParseTuple(args, "O!sO:gnome_app_insert_menus", PyGtk_Type, &app,
+  if (!PyArg_ParseTuple(args, "O!sO:gnome_app_insert_menus", &PyGtk_Type, &app,
 			&path, &list))
     return NULL;
   if ((uiinfo = list_to_ui_info(list, FALSE)) == NULL)
@@ -482,7 +372,7 @@ static PyObject *_wrap_gnome_app_install_menu_hints(PyObject *self, PyObject *ar
   PyObject *app, *list;
   GnomeUIInfo *uiinfo;
 
-  if (!PyArg_ParseTuple(args, "O!O:gnome_app_install_menu_hints", PyGtk_Type,
+  if (!PyArg_ParseTuple(args, "O!O:gnome_app_install_menu_hints", &PyGtk_Type,
 			&app, &list))
     return NULL;
   if ((uiinfo = list_to_ui_info(list, FALSE)) == NULL)
@@ -523,7 +413,7 @@ static PyObject *_wrap_gnome_canvas_get_scroll_region(PyObject *self,
 						 PyObject *args) {
   PyObject *obj;
   double x1, y1, x2, y2;
-  if (!PyArg_ParseTuple(args, "O!:gnome_canvas_get_scroll_region", PyGtk_Type,
+  if (!PyArg_ParseTuple(args, "O!:gnome_canvas_get_scroll_region", &PyGtk_Type,
 			&obj))
     return NULL;
   gnome_canvas_get_scroll_region(GNOME_CANVAS(PyGtk_Get(obj)), &x1, &y1,
@@ -535,7 +425,7 @@ static PyObject *_wrap_gnome_canvas_get_scroll_offsets(PyObject *self,
 						  PyObject *args) {
   PyObject *obj;
   int cx, cy;
-  if (!PyArg_ParseTuple(args, "O!:gnome_canvas_get_scroll_offsets", PyGtk_Type,
+  if (!PyArg_ParseTuple(args, "O!:gnome_canvas_get_scroll_offsets", &PyGtk_Type,
 			&obj))
     return NULL;
   gnome_canvas_get_scroll_offsets(GNOME_CANVAS(PyGtk_Get(obj)), &cx, &cy);
@@ -546,7 +436,7 @@ static PyObject *_wrap_gnome_canvas_w2c(PyObject *self, PyObject *args) {
   PyObject *obj;
   int cx, cy;
   double wx = 0.0, wy = 0.0;
-  if (!PyArg_ParseTuple(args, "O!dd:gnome_canvas_w2c", PyGtk_Type, &obj,wx,wy))
+  if (!PyArg_ParseTuple(args, "O!dd:gnome_canvas_w2c", &PyGtk_Type, &obj,wx,wy))
     return NULL;
   gnome_canvas_w2c(GNOME_CANVAS(PyGtk_Get(obj)), wx, wy, &cx, &cy);
   return Py_BuildValue("(ii)", cx, cy);
@@ -556,7 +446,7 @@ static PyObject *_wrap_gnome_canvas_c2w(PyObject *self, PyObject *args) {
   PyObject *obj;
   int cx = 0, cy = 0;
   double wx, wy;
-  if (!PyArg_ParseTuple(args, "O!ii:gnome_canvas_w2c", PyGtk_Type, &obj,cx,cy))
+  if (!PyArg_ParseTuple(args, "O!ii:gnome_canvas_w2c", &PyGtk_Type, &obj,cx,cy))
     return NULL;
   gnome_canvas_c2w(GNOME_CANVAS(PyGtk_Get(obj)), cx, cy, &wx, &wy);
   return Py_BuildValue("(dd)", wx, wy);
@@ -566,7 +456,7 @@ static PyObject *_wrap_gnome_canvas_get_color(PyObject *self, PyObject *args) {
   PyObject *obj;
   char *spec;
   GdkColor color;
-  if (!PyArg_ParseTuple(args, "O!s:gnome_canvas_get_color", PyGtk_Type, &obj,
+  if (!PyArg_ParseTuple(args, "O!s:gnome_canvas_get_color", &PyGtk_Type, &obj,
 			&spec));
   if (gnome_canvas_get_color(GNOME_CANVAS(PyGtk_Get(obj)), spec, &color))
     return PyGdkColor_New(&color);
@@ -579,7 +469,7 @@ static PyObject *_wrap_gnome_canvas_get_item_at(PyObject *self, PyObject *args)
     PyObject *canvas;
     GnomeCanvasItem *item;
 
-    if (!PyArg_ParseTuple(args, "O!dd:gnome_canvas_get_item_at", PyGtk_Type,
+    if (!PyArg_ParseTuple(args, "O!dd:gnome_canvas_get_item_at", &PyGtk_Type,
 			  &canvas, &x, &y))
         return NULL;
     item = gnome_canvas_get_item_at(GNOME_CANVAS(PyGtk_Get(canvas)), x, y);
@@ -594,7 +484,7 @@ static PyObject *_wrap_gnome_canvas_item_new(PyObject *self, PyObject *args) {
   GtkType type;
   gint nargs;
   GtkArg *arg;
-  if (!PyArg_ParseTuple(args, "O!iO!:gnome_canvas_item_new", PyGtk_Type,
+  if (!PyArg_ParseTuple(args, "O!iO!:gnome_canvas_item_new", &PyGtk_Type,
 			&parent, &type, &PyDict_Type, &dict))
     return NULL;
   arg = PyDict_AsGtkArgs(dict, type, &nargs);
@@ -611,7 +501,7 @@ static PyObject *_wrap_gnome_canvas_item_set(PyObject *self, PyObject *args) {
   GtkType type;
   gint nargs;
   GtkArg *arg;
-  if (!PyArg_ParseTuple(args, "O!O!:gnome_canvas_item_set", PyGtk_Type,
+  if (!PyArg_ParseTuple(args, "O!O!:gnome_canvas_item_set", &PyGtk_Type,
 			&obj, &PyDict_Type, &dict))
     return NULL;
   type = GTK_OBJECT_TYPE(PyGtk_Get(obj));
@@ -630,7 +520,7 @@ static PyObject *_wrap_gnome_canvas_item_affine_relative(PyObject *self, PyObjec
   int i;
 
   if (!PyArg_ParseTuple(args, "O!O:gnome_canvas_item_affine_relative",
-			PyGtk_Type, &item, &py_affine))
+			&PyGtk_Type, &item, &py_affine))
     return NULL;
   if (!PySequence_Check(py_affine) || PySequence_Length(py_affine) != 6) {
     PyErr_SetString(PyExc_TypeError, "second argument must be a sequence, length 6");
@@ -658,7 +548,7 @@ static PyObject *_wrap_gnome_canvas_item_affine_absolute(PyObject *self, PyObjec
   int i;
 
   if (!PyArg_ParseTuple(args, "O!O:gnome_canvas_item_affine_absolute",
-			PyGtk_Type, &item, &py_affine))
+			&PyGtk_Type, &item, &py_affine))
     return NULL;
   if (!PySequence_Check(py_affine) || PySequence_Length(py_affine) != 6) {
     PyErr_SetString(PyExc_TypeError, "second argument must be a sequence, length 6");
@@ -684,7 +574,7 @@ static PyObject *_wrap_gnome_canvas_item_w2i(PyObject *self, PyObject *args) {
   PyObject *item;
   double x, y;
 
-  if (!PyArg_ParseTuple(args, "O!dd:gnome_canvas_item_w2i", PyGtk_Type,
+  if (!PyArg_ParseTuple(args, "O!dd:gnome_canvas_item_w2i", &PyGtk_Type,
 			&item, &x, &y))
     return NULL;
   gnome_canvas_item_w2i(GNOME_CANVAS_ITEM(PyGtk_Get(item)), &x, &y);
@@ -695,7 +585,7 @@ static PyObject *_wrap_gnome_canvas_item_i2w(PyObject *self, PyObject *args) {
   PyObject *item;
   double x, y;
 
-  if (!PyArg_ParseTuple(args, "O!dd:gnome_canvas_item_i2w", PyGtk_Type,
+  if (!PyArg_ParseTuple(args, "O!dd:gnome_canvas_item_i2w", &PyGtk_Type,
 			&item, &x, &y))
     return NULL;
   gnome_canvas_item_i2w(GNOME_CANVAS_ITEM(PyGtk_Get(item)), &x, &y);
@@ -706,7 +596,7 @@ static PyObject *_wrap_gnome_canvas_item_get_bounds(PyObject *self, PyObject *ar
   PyObject *item;
   double x1, y1, x2, y2;
 
-  if (!PyArg_ParseTuple(args, "O!:gnome_canvas_item_get_bounds", PyGtk_Type,
+  if (!PyArg_ParseTuple(args, "O!:gnome_canvas_item_get_bounds", &PyGtk_Type,
 			&item))
     return NULL;
   gnome_canvas_item_get_bounds(GNOME_CANVAS_ITEM(PyGtk_Get(item)),
@@ -714,6 +604,22 @@ static PyObject *_wrap_gnome_canvas_item_get_bounds(PyObject *self, PyObject *ar
   return Py_BuildValue("(dddd)", x1, y1, x2, y2);
 }
 
+static PyObject *_wrap_gnome_canvas_group_children(PyObject *self, PyObject *args) {
+  PyObject *group, *ret;
+  GList *children;
+  
+  if (!PyArg_ParseTuple(args, "O!:gnome_canvas_group_children", &PyGtk_Type,
+			&group))
+    return NULL;
+  children = GNOME_CANVAS_GROUP(PyGtk_Get(group))->item_list;
+  ret = PyList_New(0);
+  while (children) {
+    PyList_Append(ret, PyGtk_New(children->data));
+    children = children->next;
+  }
+  return ret;
+}
+
 static int read_vector(PyObject *obj, int *argc, char ***argv) {
   int i;
   if (obj == Py_None) {
@@ -743,7 +649,7 @@ static PyObject *_wrap_gnome_client_set_clone_command(PyObject *self, PyObject *
   int argc;
   char **argv = NULL;
   if (!PyArg_ParseTuple(args, "O!|O:gnome_client_set_clone_command",
-			PyGtk_Type, &client, &av))
+			&PyGtk_Type, &client, &av))
     return NULL;
   if (read_vector(av, &argc, &argv))
     return NULL;
@@ -758,7 +664,7 @@ static PyObject *_wrap_gnome_client_set_discard_command(PyObject *self, PyObject
   int argc;
   char **argv = NULL;
   if (!PyArg_ParseTuple(args, "O!|O:gnome_client_set_discard_command",
-			PyGtk_Type, &client, &av))
+			&PyGtk_Type, &client, &av))
     return NULL;
   if (read_vector(av, &argc, &argv))
     return NULL;
@@ -773,7 +679,7 @@ static PyObject *_wrap_gnome_client_set_restart_command(PyObject *self, PyObject
   int argc;
   char **argv = NULL;
   if (!PyArg_ParseTuple(args, "O!|O:gnome_client_set_restart_command",
-			PyGtk_Type, &client, &av))
+			&PyGtk_Type, &client, &av))
     return NULL;
   if (read_vector(av, &argc, &argv))
     return NULL;
@@ -788,7 +694,7 @@ static PyObject *_wrap_gnome_client_set_resign_command(PyObject *self, PyObject
   int argc;
   char **argv = NULL;
   if (!PyArg_ParseTuple(args, "O!|O:gnome_client_set_resign_command",
-			PyGtk_Type, &client, &av))
+			&PyGtk_Type, &client, &av))
     return NULL;
   if (read_vector(av, &argc, &argv))
     return NULL;
@@ -803,7 +709,7 @@ static PyObject *_wrap_gnome_client_set_shutdown_command(PyObject *self, PyObjec
   int argc;
   char **argv = NULL;
   if (!PyArg_ParseTuple(args, "O!|O:gnome_client_set_shutdown_command",
-			PyGtk_Type, &client, &av))
+			&PyGtk_Type, &client, &av))
     return NULL;
   if (read_vector(av, &argc, &argv))
     return NULL;
@@ -845,7 +751,7 @@ static PyObject *_wrap_gnome_client_request_interaction(PyObject *self,
 							PyObject *args) {
   PyObject *client, *func, *py_dialog;
   GnomeDialogType dialog;
-  if (!PyArg_ParseTuple(args, "O!OO", PyGtk_Type, &client, &py_dialog, &func))
+  if (!PyArg_ParseTuple(args, "O!OO", &PyGtk_Type, &client, &py_dialog, &func))
     return NULL;
   if (PyGtkEnum_get_value(GTK_TYPE_GNOME_DIALOG_TYPE,py_dialog,(gint*)&dialog))
     return NULL;
@@ -866,7 +772,7 @@ static PyObject *_wrap_gnome_client_request_interaction(PyObject *self,
 static PyObject *_wrap_gnome_color_picker_get_d(PyObject *self, PyObject *args) {
   PyObject *obj;
   double r, g, b, a;
-  if (!PyArg_ParseTuple(args, "O!:gnome_color_picker_get_d", PyGtk_Type, &obj))
+  if (!PyArg_ParseTuple(args, "O!:gnome_color_picker_get_d", &PyGtk_Type, &obj))
     return NULL;
   gnome_color_picker_get_d(GNOME_COLOR_PICKER(PyGtk_Get(obj)), &r, &g, &b, &a);
   return Py_BuildValue("(dddd)", r, g, b, a);
@@ -875,7 +781,7 @@ static PyObject *_wrap_gnome_color_picker_get_d(PyObject *self, PyObject *args)
 static PyObject *_wrap_gnome_color_picker_get_i8(PyObject *self, PyObject *args) {
   PyObject *obj;
   guint8 r, g, b, a;
-  if (!PyArg_ParseTuple(args, "O!:gnome_color_picker_get_i8", PyGtk_Type,&obj))
+  if (!PyArg_ParseTuple(args, "O!:gnome_color_picker_get_i8", &PyGtk_Type,&obj))
     return NULL;
   gnome_color_picker_get_i8(GNOME_COLOR_PICKER(PyGtk_Get(obj)), &r, &g, &b,&a);
   return Py_BuildValue("(iiii)", (int)r, (int)g, (int)b, (int)a);
@@ -884,7 +790,7 @@ static PyObject *_wrap_gnome_color_picker_get_i8(PyObject *self, PyObject *args)
 static PyObject *_wrap_gnome_color_picker_get_i16(PyObject *self, PyObject *args) {
   PyObject *obj;
   gushort r, g, b, a;
-  if (!PyArg_ParseTuple(args, "O!:gnome_color_picker_get_i16",PyGtk_Type,&obj))
+  if (!PyArg_ParseTuple(args, "O!:gnome_color_picker_get_i16",&PyGtk_Type,&obj))
     return NULL;
   gnome_color_picker_get_i16(GNOME_COLOR_PICKER(PyGtk_Get(obj)), &r, &g,&b,&a);
   return Py_BuildValue("(iiii)", (int)r, (int)g, (int)b, (int)a);
@@ -894,7 +800,7 @@ static PyObject *_wrap_gnome_dentry_edit_save_file(PyObject *self, PyObject *arg
   PyObject *obj;
   char *fname;
   GnomeDesktopEntry *de;
-  if (!PyArg_ParseTuple(args, "O!s:gnome_dentry_edit_save_file", PyGtk_Type,
+  if (!PyArg_ParseTuple(args, "O!s:gnome_dentry_edit_save_file", &PyGtk_Type,
 			&obj, &fname))
     return NULL;
   de = gnome_dentry_get_dentry(GNOME_DENTRY_EDIT(PyGtk_Get(obj)));
@@ -909,7 +815,7 @@ static PyObject *_wrap_gnome_dialog_button_connect(PyObject *self, PyObject *arg
   PyObject *obj, *callback;
   int button;
 
-  if (!PyArg_ParseTuple(args, "O!iO:gnome_dialog_button_connect", PyGtk_Type,
+  if (!PyArg_ParseTuple(args, "O!iO:gnome_dialog_button_connect", &PyGtk_Type,
 			&obj, &button, &callback))
     return NULL;
   if (!PyCallable_Check(callback)) {
@@ -926,7 +832,7 @@ static PyObject *_wrap_gnome_icon_list_set_icon_data(PyObject *self,
 						     PyObject *args) {
   PyObject *ilist, *data;
   int pos;
-  if (!PyArg_ParseTuple(args, "O!iO:gnome_icon_list_set_icon_data", PyGtk_Type,
+  if (!PyArg_ParseTuple(args, "O!iO:gnome_icon_list_set_icon_data", &PyGtk_Type,
 			&ilist, &pos, &data))
     return NULL;
   Py_INCREF(data);
@@ -941,7 +847,7 @@ static PyObject *_wrap_gnome_icon_list_find_icon_from_data(PyObject *self, PyObj
   PyObject *ilist, *data;
 
   if (!PyArg_ParseTuple(args, "O!O:gnome_icon_list_find_icon_from_data",
-			PyGtk_Type, &ilist, &data))
+			&PyGtk_Type, &ilist, &data))
     return NULL;
   return PyInt_FromLong(gnome_icon_list_find_icon_from_data(
 			GNOME_ICON_LIST(PyGtk_Get(ilist)), data));
@@ -952,7 +858,7 @@ static PyObject *_wrap_gnome_icon_list_get_icon_data(PyObject *self, PyObject *a
   int pos;
 
   if (!PyArg_ParseTuple(args, "O!i:gnome_icon_list_get_icon_data",
-			PyGtk_Type, &ilist, &pos))
+			&PyGtk_Type, &ilist, &pos))
     return NULL;
   data = gnome_icon_list_get_icon_data(GNOME_ICON_LIST(PyGtk_Get(ilist)), pos);
   Py_INCREF(data);
@@ -963,7 +869,7 @@ static PyObject *_wrap_gnome_mdi_set_menubar_template(PyObject *self, PyObject *
   PyObject *mdi, *list;
   GnomeUIInfo *uiinfo;
 
-  if (!PyArg_ParseTuple(args, "O!O:gnome_mdi_set_menubar_template", PyGtk_Type,
+  if (!PyArg_ParseTuple(args, "O!O:gnome_mdi_set_menubar_template", &PyGtk_Type,
 			&mdi, &list))
     return NULL;
   if ((uiinfo = list_to_ui_info(list, TRUE)) == NULL)
@@ -978,7 +884,7 @@ static PyObject *_wrap_gnome_mdi_set_toolbar_template(PyObject *self, PyObject *
   PyObject *mdi, *list;
   GnomeUIInfo *uiinfo;
 
-  if (!PyArg_ParseTuple(args, "O!O:gnome_mdi_set_toolbar_template", PyGtk_Type,
+  if (!PyArg_ParseTuple(args, "O!O:gnome_mdi_set_toolbar_template", &PyGtk_Type,
 			&mdi, &list))
     return NULL;
   if ((uiinfo = list_to_ui_info(list, TRUE)) == NULL)
@@ -994,7 +900,7 @@ static PyObject *_wrap_gnome_mdi_child_set_menu_template(PyObject *self, PyObjec
   GnomeUIInfo *uiinfo;
 
   if (!PyArg_ParseTuple(args, "O!O:gnome_mdi_child_set_menu_template",
-			PyGtk_Type, &mdi, &list))
+			&PyGtk_Type, &mdi, &list))
     return NULL;
   if ((uiinfo = list_to_ui_info(list, TRUE)) == NULL)
     return NULL;
@@ -1008,7 +914,7 @@ static PyObject *_wrap_gnome_mdi_generic_child_set_view_creator(PyObject *self,
   PyObject *mdi, *func, *data = NULL;
 
   if (!PyArg_ParseTuple(args, "O!O|O:gnome_mdi_generic_child_set_view_creator",
-			PyGtk_Type, &mdi, &func, &data))
+			&PyGtk_Type, &mdi, &func, &data))
     return NULL;
   if (!PyCallable_Check(func)) {
     PyErr_SetString(PyExc_TypeError, "second argument must be callable");
@@ -1035,7 +941,7 @@ static PyObject *_wrap_gnome_mdi_generic_child_set_menu_creator(PyObject *self,
   PyObject *mdi, *func, *data = NULL;
 
   if (!PyArg_ParseTuple(args, "O!O|O:gnome_mdi_generic_child_set_menu_creator",
-			PyGtk_Type, &mdi, &func, &data))
+			&PyGtk_Type, &mdi, &func, &data))
     return NULL;
   if (!PyCallable_Check(func)) {
     PyErr_SetString(PyExc_TypeError, "second argument must be callable");
@@ -1062,7 +968,7 @@ static PyObject *_wrap_gnome_mdi_generic_child_set_config_func(PyObject *self, P
   PyObject *mdi, *func, *data = NULL;
 
   if (!PyArg_ParseTuple(args, "O!O|O:gnome_mdi_generic_child_set_config_func",
-			PyGtk_Type, &mdi, &func, &data))
+			&PyGtk_Type, &mdi, &func, &data))
     return NULL;
   if (!PyCallable_Check(func)) {
     PyErr_SetString(PyExc_TypeError, "second argument must be callable");
@@ -1089,7 +995,7 @@ static PyObject *_wrap_gnome_mdi_generic_child_set_label_func(PyObject *self, Py
   PyObject *mdi, *func, *data = NULL;
 
   if (!PyArg_ParseTuple(args, "O!O|O:gnome_mdi_generic_child_set_label_func",
-			PyGtk_Type, &mdi, &func, &data))
+			&PyGtk_Type, &mdi, &func, &data))
     return NULL;
   if (!PyCallable_Check(func)) {
     PyErr_SetString(PyExc_TypeError, "second argument must be callable");
@@ -1130,8 +1036,8 @@ static PyObject *_wrap_gnome_popup_menu_new(PyObject *self, PyObject *args) {
 static PyObject *_wrap_gnome_popup_menu_attach(PyObject *self,PyObject *args) {
   PyObject *popup, *widget;
 
-  if (!PyArg_ParseTuple(args, "O!O!:gnome_popup_menu_attach", PyGtk_Type,
-			&popup, PyGtk_Type, &widget))
+  if (!PyArg_ParseTuple(args, "O!O!:gnome_popup_menu_attach", &PyGtk_Type,
+			&popup, &PyGtk_Type, &widget))
     return NULL;
   gnome_popup_menu_attach(GTK_WIDGET(PyGtk_Get(popup)),
 			  GTK_WIDGET(PyGtk_Get(widget)), NULL);
@@ -1146,7 +1052,7 @@ static PyObject *_wrap_gnome_widget_add_help_with_uidata(PyObject *self,
   GnomeUIInfo *menuinfo;
 
   if (!PyArg_ParseTuple(args, "O!sO:gnome_widget_add_help_with_uidata",
-			PyGtk_Type, &widget, &help, &list))
+			&PyGtk_Type, &widget, &help, &list))
     return NULL;
   if ((menuinfo = list_to_ui_info(list, TRUE)) == NULL)
     return NULL;
@@ -1460,7 +1366,7 @@ static PyObject *_wrap_gnome_question_dialog_parented(PyObject *self, PyObject *
   PyObject *callback, *parent;
 
   if (!PyArg_ParseTuple(args, "sOO!:gnome_question_dialog_parented", &string,
-			&callback, PyGtk_Type, &parent))
+			&callback, &PyGtk_Type, &parent))
     return NULL;
   if (!PyCallable_Check(callback)) {
     PyErr_SetString(PyExc_TypeError, "second argument must be callable");
@@ -1495,7 +1401,7 @@ static PyObject *_wrap_gnome_ok_cancel_dialog_parented(PyObject *self, PyObject
   PyObject *callback, *parent;
 
   if (!PyArg_ParseTuple(args, "sOO!:gnome_ok_cancel_dialog_parented", &string,
-			&callback, PyGtk_Type, &parent))
+			&callback, &PyGtk_Type, &parent))
     return NULL;
   if (!PyCallable_Check(callback)) {
     PyErr_SetString(PyExc_TypeError, "second argument must be callable");
@@ -1538,7 +1444,7 @@ static PyObject *_wrap_gnome_app_question(PyObject *self, PyObject *args) {
   PyObject *app, *callback;
   char *question;
 
-  if (!PyArg_ParseTuple(args, "O!sO:gnome_app_question", PyGtk_Type, &app,
+  if (!PyArg_ParseTuple(args, "O!sO:gnome_app_question", &PyGtk_Type, &app,
 			&question, &callback))
     return NULL;
   if (!PyCallable_Check(callback)) {
@@ -1556,7 +1462,7 @@ static PyObject *_wrap_gnome_app_ok_cancel(PyObject *self, PyObject *args) {
   PyObject *app, *callback;
   char *question;
 
-  if (!PyArg_ParseTuple(args, "O!sO:gnome_app_ok_cancel", PyGtk_Type, &app,
+  if (!PyArg_ParseTuple(args, "O!sO:gnome_app_ok_cancel", &PyGtk_Type, &app,
 			&question, &callback))
     return NULL;
   if (!PyCallable_Check(callback)) {
@@ -1574,7 +1480,7 @@ static PyObject *_wrap_gnome_app_request_string(PyObject *self, PyObject *args)
   PyObject *app, *callback;
   char *question;
 
-  if (!PyArg_ParseTuple(args, "O!sO:gnome_app_request_string", PyGtk_Type,
+  if (!PyArg_ParseTuple(args, "O!sO:gnome_app_request_string", &PyGtk_Type,
 			&app, &question, &callback))
     return NULL;
   if (!PyCallable_Check(callback)) {
@@ -1592,7 +1498,7 @@ static PyObject *_wrap_gnome_app_request_password(PyObject *self, PyObject *args
   PyObject *app, *callback;
   char *question;
 
-  if (!PyArg_ParseTuple(args, "O!sO:gnome_app_request_password", PyGtk_Type,
+  if (!PyArg_ParseTuple(args, "O!sO:gnome_app_request_password", &PyGtk_Type,
 			&app, &question, &callback))
     return NULL;
   if (!PyCallable_Check(callback)) {
@@ -1628,6 +1534,10 @@ static PyMethodDef gnomeuiMethods[] = {
     { "gnome_canvas_item_set", _wrap_gnome_canvas_item_set, 1 },
     { "gnome_canvas_item_affine_relative", _wrap_gnome_canvas_item_affine_relative, 1 },
     { "gnome_canvas_item_affine_absolute", _wrap_gnome_canvas_item_affine_absolute, 1 },
+    { "gnome_canvas_item_i2w", _wrap_gnome_canvas_item_i2w, 1 },
+    { "gnome_canvas_item_w2i", _wrap_gnome_canvas_item_w2i, 1 },
+    { "gnome_canvas_item_get_bounds", _wrap_gnome_canvas_item_get_bounds, 1 },
+    { "gnome_canvas_group_children", _wrap_gnome_canvas_group_children, 1 },
     { "gnome_client_set_clone_command", _wrap_gnome_client_set_clone_command, 1 },
     { "gnome_client_set_discard_command", _wrap_gnome_client_set_discard_command, 1 },
     { "gnome_client_set_restart_command", _wrap_gnome_client_set_restart_command, 1 },
@@ -1673,96 +1583,8 @@ void init_gnomeui() {
 
     m = Py_InitModule("_gnomeui", gnomeuiMethods);
 
-    m = PyImport_ImportModule("_gtk");
-    if (m == NULL) {
-        Py_FatalError("couldn't import _gtk");
-        return;
-    }
-    d = PyModule_GetDict(m);
-    Py_DECREF(m);
-
-    PyGtk_Type = (PyTypeObject *)PyDict_GetItemString(d, "GtkObjectType");
-    Py_INCREF(PyGtk_Type);
-    PyGtkAccelGroup_Type = (PyTypeObject *)PyDict_GetItemString(d, "GtkAccelGroupType");
-    Py_INCREF(PyGtkAccelGroup_Type);
-    PyGdkFont_Type = (PyTypeObject *)PyDict_GetItemString(d, "GdkFontType");
-    Py_INCREF(PyGdkFont_Type);
-    PyGdkColor_Type = (PyTypeObject *)PyDict_GetItemString(d, "GdkColorType");
-    Py_INCREF(PyGdkColor_Type);
-    PyGdkEvent_Type = (PyTypeObject *)PyDict_GetItemString(d, "GdkEventType");
-    Py_INCREF(PyGdkEvent_Type);
-    PyGdkWindow_Type = (PyTypeObject *)PyDict_GetItemString(d,"GdkWindowType");
-    Py_INCREF(PyGdkWindow_Type);
-    PyGdkGC_Type = (PyTypeObject *)PyDict_GetItemString(d, "GdkGCType");
-    Py_INCREF(PyGdkGC_Type);
-    PyGdkColormap_Type = (PyTypeObject *)PyDict_GetItemString(d,
-						"GdkColormapType");
-    Py_INCREF(PyGdkColormap_Type);
-    PyGdkCursor_Type = (PyTypeObject *)PyDict_GetItemString(d,"GdkCursorType");
-    Py_INCREF(PyGdkCursor_Type);
-
-    d = PyDict_GetItemString(d, "_private");
-    PyGtk_New = PyCObject_AsVoidPtr(PyDict_GetItemString(d, "PyGtk_New"));
-    PyGtkAccelGroup_New = PyCObject_AsVoidPtr(PyDict_GetItemString(d,
-						"PyGtkAccelGroup_New"));
-    PyGdkFont_New = PyCObject_AsVoidPtr(PyDict_GetItemString(d,
-						"PyGdkFont_New"));
-    PyGdkColor_New = PyCObject_AsVoidPtr(PyDict_GetItemString(d,
-						"PyGdkColor_New"));
-    PyGdkEvent_New = PyCObject_AsVoidPtr(PyDict_GetItemString(d,
-						"PyGdkEvent_New"));
-    PyGdkWindow_New = PyCObject_AsVoidPtr(PyDict_GetItemString(d,
-						"PyGdkWindow_New"));
-    PyGdkGC_New = PyCObject_AsVoidPtr(PyDict_GetItemString(d, "PyGdkGC_New"));
-    PyGdkColormap_New = PyCObject_AsVoidPtr(PyDict_GetItemString(d,
-						"PyGdkColormap_New"));
-    PyGdkCursor_New = PyCObject_AsVoidPtr(PyDict_GetItemString(d,
-						"PyGdkCursor_New"));
-    PyGtk_DestroyNotify = PyCObject_AsVoidPtr(PyDict_GetItemString(d,
-						"PyGtk_DestroyNotify"));
-    PyGtk_CallbackMarshal = PyCObject_AsVoidPtr(PyDict_GetItemString(d,
-						"PyGtk_CallbackMarshal"));
-    PyGtkFlag_get_value = PyCObject_AsVoidPtr(PyDict_GetItemString(d,
-						"PyGtkFlag_get_value"));
-    PyGtkEnum_get_value = PyCObject_AsVoidPtr(PyDict_GetItemString(d,
-						"PyGtkEnum_get_value"));
-    PyDict_AsGtkArgs = PyCObject_AsVoidPtr(PyDict_GetItemString(d,
-						"PyDict_AsGtkArgs"));
-    PyGtk_RegisterBoxed = PyCObject_AsVoidPtr(PyDict_GetItemString(d,
-						"PyGtk_RegisterBoxed"));
-    PyGtk_BlockThreads = PyCObject_AsVoidPtr(
-                        PyDict_GetItemString(d, "PyGtk_BlockThreads"));
-    PyGtk_UnblockThreads = PyCObject_AsVoidPtr(
-                        PyDict_GetItemString(d, "PyGtk_UnblockThreads"));
-
-    m = PyImport_ImportModule("_gdkimlib");
-    if (m == NULL) {
-        Py_FatalError("couldn't import _gdkimlib");
-        return;
-    }
-    d = PyModule_GetDict(m);
-    Py_DECREF(m);
-    PyGdkImlibImage_Type = (PyTypeObject *)PyDict_GetItemString(d,
-						"GdkImlibImageType");
-    Py_INCREF(PyGdkImlibImage_Type);
-
-    d = PyDict_GetItemString(d, "_private");
-    PyGdkImlibImage_New = PyCObject_AsVoidPtr(PyDict_GetItemString(d,
-						"PyGdkImlibImage_New"));
-
-     m = PyImport_ImportModule("os");
-     if (m == NULL) {
-         Py_FatalError("couldn't import os");
-         return;
-     }
-     d = PyModule_GetDict(m);
-     Py_DECREF(m);
-     d = PyDict_GetItemString(d, "environ");
-     d = PyMapping_GetItemString(d, "PYGTK_FATAL_EXCEPTIONS");
-     if (d == NULL)
-         PyErr_Clear();
-     else
-         PyGtk_FatalExceptions = PyObject_IsTrue(d);
+    init_pygtk();
+    init_pygdkimlib();
 
     if (PyErr_Occurred())
         Py_FatalError("can't initialise module _gnomeui");
